<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Repository;

use DateTimeInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteStateEnum;
use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Component\Channel\Model\ChannelInterface;
use Sylius\Component\Customer\Model\CustomerInterface;

class QuoteRepository extends EntityRepository implements QuoteRepositoryInterface
{
    public function findQuoteForSummary(int $id): ?QuoteInterface
    {
        return $this->createQueryBuilder('o')
            ->andWhere('o.id = :id')
            ->andWhere('o.state = :state')
            ->setParameter('id', $id)
            ->setParameter('state', QuoteStateEnum::STATE_NEW)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    /**
     * @throws NonUniqueResultException
     */
    public function findQuoteByChannel(int $id, ChannelInterface $channel): ?QuoteInterface
    {
        return $this->createQueryBuilder('o')
            ->andWhere('o.id = :id')
            ->andWhere('o.state = :state')
            ->andWhere('o.channel = :channel')
            ->setParameter('id', $id)
            ->setParameter('state', QuoteStateEnum::STATE_NEW)
            ->setParameter('channel', $channel)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    public function findLatestQuoteByChannelAndCustomer(ChannelInterface $channel, CustomerInterface $customer): ?QuoteInterface
    {
        return $this->createQueryBuilder('o')
            ->andWhere('o.state = :state')
            ->andWhere('o.channel = :channel')
            ->andWhere('o.customer = :customer')
            ->setParameter('state', QuoteStateEnum::STATE_NEW)
            ->setParameter('channel', $channel)
            ->setParameter('customer', $customer)
            ->addOrderBy('o.createdAt', 'DESC')
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    /**
     * @throws NonUniqueResultException
     */
    public function findOneByNumberAndCustomer(string $number, CustomerInterface $customer): ?QuoteInterface
    {
        return $this->createSubmittedQuotesQueryBuilder()
            ->andWhere('o.number = :number')
            ->andWhere('o.customer = :customer')
            ->setParameter('number', $number)
            ->setParameter('customer', $customer)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    public function createSubmittedQuotesQueryBuilder(string $alias = 'o'): QueryBuilder
    {
        return $this->createQueryBuilder($alias)
            ->andWhere(sprintf('%s.state != :state', $alias))
            ->setParameter('state', QuoteStateEnum::STATE_NEW)
        ;
    }

    public function findQuoteById(int $id): ?QuoteInterface
    {
        return $this->createQueryBuilder('o')
            ->andWhere('o.id = :id')
            ->andWhere('o.state = :state')
            ->setParameter('state', QuoteStateEnum::STATE_NEW)
            ->setParameter('id', $id)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }

    public function createByCustomerAndChannelIdQueryBuilder(int $customerId, int $channelId): QueryBuilder
    {
        return $this->createSubmittedQuotesQueryBuilder()
            ->andWhere('o.customer = :customerId')
            ->andWhere('o.channel = :channelId')
            ->setParameter('customerId', $customerId)
            ->setParameter('channelId', $channelId)
        ;
    }

    public function findExpiredQuotes(DateTimeInterface $expirationDate): iterable
    {
        $queryBuilder = $this->createQueryBuilder('o');

        return $queryBuilder
            ->andWhere('o.expirationDate = :expirationDate')
            ->andWhere($queryBuilder->expr()->notIn('o.state', [QuoteStateEnum::STATE_NEW, QuoteStateEnum::STATE_EXPIRED]))
            ->setParameter('expirationDate', $expirationDate->format('Y-m-d'))
            ->getQuery()
            ->getResult()
        ;
    }
}
